/* ICONSOLE.sc

   Input driver for DAMP by Klaus Peichl.
   This driver reads its input from a simple keypad connected to
   the parallel port. Each of the 5 status lines of the port must be
   connected to a contact switch which grounds the line when
   being pressed.

   Key mapping:

   Name of key       LPT-pin   function
   ------------------------------------
   key 1               13       Pause
   key 2               12        <<
   key 3               11        >>
   key 4               10       STOP (simply quits from DAMP)
   menu key            15       MENU (simply quits from DAMP, too)

*/

/* We MUST have the next line: */
#include "dampinp.sh"


/* Now we'll define some constants */

const short LPT_DATA = 0x378;
const short LPT_STATUS = 0x379;
const short LPT_CONTROL = 0x37a;
const unsigned char zero = 0;

unsigned char  Status=255;
unsigned char  lastStatus=0;


/*===========================================================================
  input_driver_init()

  This function is called when DAMP starts up, and should initialise
  the input device appropriately.
===========================================================================*/

void input_driver_init()
{
   /* If damp_developer is non-zero, we should output debugging
      information */

   if(damp_developer)
      printf("input_driver_init(): Beginning initialisation.\n");

   /* The very first thing to do, is setup all the driver information */

   sprintf(input_driver_name,"LPT Input Driver");
   sprintf(input_driver_author,"Klaus Peichl");
   sprintf(input_driver_author_email,"pei@01019freenet.de");
   sprintf(input_driver_description,"Input driver for 5-key low-active input from LPT1");
   input_driver_version = 1.0;

   /* Then we want to remove the DAMP low-level keyboard driver. Note
      that this will NOT be the case for most other drivers, just ones
      that use the keyboard. */

      /* remove_keyboard(); */

   /* The idle level of the status signals should be HIGH.
      If this is not the case anyway, pull-up resistors can be
      connected to +5V. The next line makes Pin 14 of LPT1 high
      so you could fetch the +5V from there: */

   outportb(LPT_CONTROL,zero);


   if(damp_developer)
      printf("input_driver_init(): Initialisation successful.\n");
}

/*===========================================================================
  input_driver_poll()

  This is called regularly by DAMP.  It should check for input and call
  the appropriate util_* function to perform the desired action.

  It should set input_driver_return_value to the value that the
  util_* function returns, or 0 if you don't call any util_* functions.
===========================================================================*/

void input_driver_poll()
{
   input_driver_return_value = 0;
   lastStatus = Status;

   // Invert all signals but 'busy' so that all signals appear
   // high-active in the variable 'Status':
   Status = inportb(LPT_STATUS) ^ 0x78;


   // For some keys, it is checked if their bit has been 0 before
   // so that the corresponding function is called only once the
   // key is pressed.
   // For some other functions (rewind and forward), it is nicer
   // to have an auto-repeat feature, so that check is not done there.


   if ( (Status & 16) && !(lastStatus & 16) )  // first key
   {
      input_driver_return_value = util_pause();
   }


   if ( (Status & 32) )  // second key
   {
      input_driver_return_value = util_rewind();
   }


   if ( (Status & 128) )  // third key
   {
      input_driver_return_value = util_fast_forward();
   }


   if ( (Status & 64) && !(lastStatus & 64) )  // fourth key
   {
      input_driver_return_value = util_quit();
   }


   if ( (Status & 8) && !(lastStatus & 8) )  // menu key
   {
      input_driver_return_value = util_quit();
   }

}
